#pragma once

#include "Array.h"

struct ID3D10Blob;
struct ID3D11PixelShader;
struct ID3D11VertexShader;
struct ID3D11Device;
struct ID3D11DeviceContext;
struct D3D11_INPUT_ELEMENT_DESC;
struct ID3D11Buffer;
struct ID3D11InputLayout;

struct ID3D12Device2;
struct ID3D12GraphicsCommandList2;
struct D3D12_INPUT_ELEMENT_DESC;
struct D3D12_ROOT_PARAMETER1;
struct D3D12_CONSTANT_BUFFER_VIEW_DESC;

namespace Framework
{
    class Text;
    class DXBuffer;
    class DX12CopyCommandQueue;
    class DX12DirectCommandQueue;

    enum ShaderType
    {
        UNBEKANNT,
        VERTEX,
        PIXEL
    };

    //! Eine Shader Klasse, die alle Constanten Buffers eines Shaders verwaltet
    class Shader : public virtual ReferenceCounter
    {
    protected:
        ShaderType type;
        RCArray< DXBuffer > *constBuffers;

    public:
        //! Konstruktor
        Shader();
        //! Destruktor
        virtual ~Shader();
        //! Setzt den Compilierten Shader
        //!  zD3d11Device: Das Device, mit welchem der Shader erstellt werden soll
        //! \param bytes Die Bytes des compilierten codes
        //! \param length die Lnge des bytearrays
        //! \return true, wenn bytes gltig ist, false sonst
        virtual bool setCompiledByteArray( unsigned char *bytes, int length ) = 0;
        //! Nach dem Aufruf dieser Funktion wird dieser Shader als Pixel Shader benutzt
        //!  zD3d11Context: Das Context Objekt, mit dem der Shader verwendet werden soll
        virtual void benutzeShader() = 0;
        //! erstellt ein constanten Buffer, der constante daten an den Shader bergibt
        //! es knnen maximal 14 Buffer erstellt werden
        //!  zD3d11Device: Das Device, mit dem der Buffer erstellt werden soll
        //! \param size Die gre des buffers in byte
        //! \param index Die position des Buffers im Buffer Array. Bereits vorhanderner Buffer wird ersetzt. Buffer 1 kann nicht erstellt werden, wenn Buffer 0 noch nicht erstellt wurde usw.
        virtual bool erstelleConstBuffer( int size, int index ) = 0;
        //! Lscht einen constanten Buffer
        //! \param index der Index des Buffers, der gelscht werden soll. Buffer 0 kann nicht gelscht werden, solange Buffer 1 noch existiert usw.
        bool removeConstBuffer( int index );
        //! Kopiert daten in einen constanten buffer
        //!  zD3d11Context: Das Context Objekt, das zum kopieren verwendt werden soll
        //! \param data Einen zeiger auf en byte Array der gre des Buffers
        //! \param index Der Index des Buffers
        //! \param len Die Lnge der Daten in Bytes (-1 fr die maximale gre des Buffers)
        bool fllConstBuffer( char *data, int index, int len = -1 );
        //! Gibt die Lnge eines constanten Buffers zurck
        //! \param index Der Index des Buffers
        int getConstBufferLnge( int index ) const;
        //! Gibt den Shadertyp zurck
        ShaderType getType() const;
    };

    class DX11Shader : public Shader
    {
    protected:
        ID3D11Device *device;
        ID3D11DeviceContext *context;

    public:
        DX11Shader( ID3D11Device *device, ID3D11DeviceContext *context );
        virtual ~DX11Shader();
        //! erstellt ein constanten Buffer, der constante daten an den Shader bergibt
        //! es knnen maximal 14 Buffer erstellt werden
        //!  zD3d11Device: Das Device, mit dem der Buffer erstellt werden soll
        //! \param size Die gre des buffers in byte
        //! \param index Die position des Buffers im Buffer Array. Bereits vorhanderner Buffer wird ersetzt. Buffer 1 kann nicht erstellt werden, wenn Buffer 0 noch nicht erstellt wurde usw.
        virtual bool erstelleConstBuffer( int size, int index ) override;
    };

    //! Verwaltet einen Pixel Shader
    class DX11PixelShader : public DX11Shader
    {
    private:
        ID3D11PixelShader *pixelShader;

    public:
        //! Konstruktor
        DX11PixelShader( ID3D11Device *device, ID3D11DeviceContext *context );
        //! Destruktor
        ~DX11PixelShader();
        //! Setzt den Compilierten Shader
        //!  zD3d11Device: Das Device, mit welchem der Shader erstellt werden soll
        //! \param bytes Die Bytes des compilierten codes
        //! \param length die Lnge des bytearrays
        //! \return true, wenn bytes gltig ist, false sonst
        bool setCompiledByteArray( unsigned char *bytes, int length ) override;
        //! Nach dem Aufruf dieser Funktion wird dieser Shader als Pixel Shader benutzt
        //!  zD3d11Context: Das Context Objekt, mit dem der Shader verwendet werden soll
        void benutzeShader() override;
    };

    //! Verwaltet einen Vertex Shader
    class DX11VertexShader : public DX11Shader
    {
    private:
        ID3D11VertexShader *vertexShader;
        ID3D11InputLayout *inputLayout;
        unsigned char *shaderByteBuffer;
        int byteBufferSize;

    public:
        //! Konstruktor
        DX11VertexShader( ID3D11Device *device, ID3D11DeviceContext *context );
        //! Destruktor
        ~DX11VertexShader();
        //! Setzt den Compilierten Shader
        //!  zD3d11Device: Das Device, mit welchem der Shader erstellt werden soll
        //! \param bytes Die Bytes des compilierten codes
        //! \param length die Lnge des bytearrays
        //! \return true, wenn bytes gltig ist, false sonst
        bool setCompiledByteArray( unsigned char *bytes, int length ) override;
        //! erstellt ein InputLayout fr den Shader
        //! Darf erst nach compile aufgerufen werden
        //!  zD3d11Device: Das Device, mit dem das Layout erstellt werden soll
        //! \param descArray Ein Array mit initialisierungsdaten
        //! \param anz Die Anzahl der Elemente im Array
        bool erstelleInputLayout( D3D11_INPUT_ELEMENT_DESC *descArray, int anz );
        //! Nach dem Aufruf dieser Funktion wird dieser Shader als Vertex Shader benutzt
        //!  zD3d11Context: Das Context Objekt, mit dem der Shader verwendet werden soll
        void benutzeShader() override;
    };

    class DX12Shader : public Shader
    {
    protected:
        ID3D12Device2 *device;
        DX12CopyCommandQueue *copy;
        DX12DirectCommandQueue *direct;
        unsigned char *shaderByteBuffer;
        int byteBufferSize;

    public:
        DX12Shader( ID3D12Device2 *device, DX12CopyCommandQueue *copy, DX12DirectCommandQueue *direct );
        virtual ~DX12Shader();
        //! erstellt ein constanten Buffer, der constante daten an den Shader bergibt
        //! es knnen maximal 14 Buffer erstellt werden
        //!  zD3d11Device: Das Device, mit dem der Buffer erstellt werden soll
        //! \param size Die gre des buffers in byte
        //! \param index Die position des Buffers im Buffer Array. Bereits vorhanderner Buffer wird ersetzt. Buffer 1 kann nicht erstellt werden, wenn Buffer 0 noch nicht erstellt wurde usw.
        virtual bool erstelleConstBuffer( int size, int index ) override;
        //! Setzt den Compilierten Shader
        //!  zD3d11Device: Das Device, mit welchem der Shader erstellt werden soll
        //! \param bytes Die Bytes des compilierten codes
        //! \param length die Lnge des bytearrays
        //! \return true, wenn bytes gltig ist, false sonst
        bool setCompiledByteArray( unsigned char *bytes, int length ) override;
        //! Nach dem Aufruf dieser Funktion wird dieser Shader als Pixel Shader benutzt
        //!  zD3d11Context: Das Context Objekt, mit dem der Shader verwendet werden soll
        void benutzeShader() override;
        //! gibt die compilierten bytes zurck
        unsigned char *getCompiledShader() const;
        //! gitbt die anzahl der compilierten bytes zurck
        int getCompiledLength() const;
        //! Erstellt den RootParameter zu einem constanten buffer
        //! \param index Der Index des Buffers
        //! \param view enthlt nach dem Aufruf die position und gre des buffers im speicher
        virtual void getViewDesc( int index, D3D12_CONSTANT_BUFFER_VIEW_DESC &view );
    };

    class DX12PixelShader : public DX12Shader
    {
    public:
        DX12PixelShader( ID3D12Device2 *device, DX12CopyCommandQueue *copy, DX12DirectCommandQueue *direct );
    };

    class DX12VertexShader : public DX12Shader
    {
    private:
        D3D12_INPUT_ELEMENT_DESC *inputLayout;
        int inputLayoutSize;

    public:
        //! Konstruktor
        DX12VertexShader( ID3D12Device2 *device, DX12CopyCommandQueue *copy, DX12DirectCommandQueue *direct );
        //! Destruktor
        ~DX12VertexShader();
        //! erstellt ein InputLayout fr den Shader
        //! Darf erst nach compile aufgerufen werden
        //!  zD3d11Device: Das Device, mit dem das Layout erstellt werden soll
        //! \param descArray Ein Array mit initialisierungsdaten
        //! \param anz Die Anzahl der Elemente im Array
        bool erstelleInputLayout( D3D12_INPUT_ELEMENT_DESC *descArray, int anz );
        //! Gint die Anzahl an eingabeparametern des Shaders zurck
        int getInputLayoutSize() const;
        //! Gibt eine Liste mit formaten fr jeden Eingabewert zurck
        D3D12_INPUT_ELEMENT_DESC *zInputLayout() const;
    };
}